/**
 * [keystroke] Generates mouse and keyboard events on Linux
 * @author Patrice Colet <pat@mamalala.org>
 * @license GNU Public License    )c( 2007
 */
 
#include <windows.h>
#include "m_pd.h"

/** The class */
t_class *kbdstroke_class;
 
typedef struct kbdstroke {
  t_object x_obj; /* The instance. Contains inlets and outlets */
  t_float xsize_factor;
  t_float ysize_factor;
  
} t_kbdstroke;



void kbdstroke_downkey(t_kbdstroke *x, t_float *s, int argc, t_atom *argv) {
  if (argc >= 1) {
    if (argv[0].a_type == A_FLOAT) {
      t_float str = atom_getfloat(argv);	  
//only use the first argument  <---fix this	  
      keybd_event(str, 0, 0, 0); 
    }
  	else {
      post("Error : Bad argument type. Must be a float or a symbol. ");
    }
  } else {
    post("Error : Missing argument");
  }
}

void kbdstroke_upkey(t_kbdstroke *x, t_float *s, int argc, t_atom *argv) {
  if (argc >= 1) {
    if (argv[0].a_type == A_FLOAT) {
      t_float str = atom_getfloat(argv);	  
//only use the first argument  <---fix this	  
      keybd_event(str, 0, 2, 0); 
    }
  	else {
      post("Error : Bad argument type. Must be a float or a symbol. ");
    }
  } else {
    post("Error : Missing argument");
  }
}
void kbdstroke_key(t_kbdstroke *x, t_float *s, int argc, t_atom *argv) {
  if (argc >= 1) {
    if (argv[0].a_type == A_FLOAT) {
      t_float str = atom_getfloat(argv);	  
//only use the first argument  <---fix this	  
    keybd_event(str, 0, 0, 0); 
    keybd_event(str, 0, 2, 0);
  
   

    }
  	else {
      post("Error : Bad argument type. Must be a float or a symbol. ");
    }
  } else {
    post("Error : Missing argument");
  }
}


void kbdstroke_mouse_move(t_kbdstroke *x, t_float *s, int argc, t_atom *argv) {
    POINT p_curseur;
      if (argc >= 2) {
          if ((argv[0].a_type == A_FLOAT)&&(argv[1].a_type == A_FLOAT))  {
          GetCursorPos(&p_curseur);
             t_float str1 = round(0.5+(p_curseur.x + round(atom_getfloatarg(0, argc, argv))) * x->xsize_factor);	  
             t_float str2 = round(0.5+(p_curseur.y + round(atom_getfloatarg(1, argc, argv))) * x->ysize_factor);   
                  
         mouse_event((MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE), str1, str2, 0 ,0);	
                
    }
  	else {
      post("Error : Bad argument type. Must be a float or a symbol. ");
    }
  } else {
    post("Error : Missing argument");
  }
}


void kbdstroke_mouse_pos(t_kbdstroke *x, t_float *s, int argc, t_atom *argv) {
  if (argc >= 2) {
    if ((argv[0].a_type == A_FLOAT)&&(argv[1].a_type == A_FLOAT))  {
      t_float str1 = atom_getfloatarg(0, argc, argv) * x->xsize_factor;	  
      t_float str2 = atom_getfloatarg(1, argc, argv) * x->ysize_factor;
      mouse_event((MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_MOVE), str1, str2, 0 ,0);
     
    }
  	else {
      post("Error : Bad argument type. Must be a float or a symbol. ");
    }
  } else {
    post("Error : Missing argument");
  }
}

void kbdstroke_mouse_wheel(t_kbdstroke *x, t_float *s, int argc, t_atom *argv) {
  if (argc >= 1) {
    if (argv[0].a_type == A_FLOAT) {
      t_float str = atom_getfloat(argv);	  
      mouse_event(MOUSEEVENTF_WHEEL, 0, 0, str ,0);	
    }
  	else {
      post("Error : Bad argument type. Must be a float or a symbol. ");
    }
  } else {
    post("Error : Missing argument");
  }
}

void kbdstroke_mouse_press(t_kbdstroke *x, t_float *s, int argc, t_atom *argv) {
  if (argc >= 1) {
    if (argv[0].a_type == A_FLOAT) {
      t_float str = atom_getfloat(argv);	  
      mouse_event(MOUSEEVENTF_WHEEL, 0, 0, str ,0);	
    }
  	else {
      post("Error : Bad argument type. Must be a float or a symbol. ");
    }
  } else {
    post("Error : Missing argument");
  }
}

void kbdstroke_mouse_click(t_kbdstroke *x, t_float *s, int argc, t_atom *argv) {
  if (argc >= 1) {
    if (argv[0].a_type == A_FLOAT) {
      t_float str = atom_getfloat(argv);	  
      mouse_event(MOUSEEVENTF_WHEEL, 0, 0, str ,0);	
    }
  	else {
      post("Error : Bad argument type. Must be a float or a symbol. ");
    }
  } else {
    post("Error : Missing argument");
  }
}

void kbdstroke_get_screen_xsize(t_kbdstroke *x, t_float *s, int argc, t_atom *argv) {
     
    POINT p_curseur;
    GetCursorPos(&p_curseur);
    outlet_float(x->x_obj.ob_outlet, x->ysize_factor);
    
}




void kbdstroke_float(t_kbdstroke *x, t_floatarg f) {
  outlet_float(x->x_obj.ob_outlet, f);
    post("appel_de_la_methode_kbdstroke_float...");
 
}

/* constructor */
void *kbdstroke_new(t_symbol *selector, int argc, t_atom *argv) {
  t_kbdstroke *x = (t_kbdstroke *) pd_new(kbdstroke_class);
  outlet_new(&x->x_obj, gensym("float"));
  x->xsize_factor = (float)65536 / (GetSystemMetrics(SM_CXSCREEN));
  x->ysize_factor = (float)65536 / (GetSystemMetrics(SM_CYSCREEN));
 
  return (void *)x;
}

/* setup */
__declspec(dllexport) void kbdstroke_setup(void) {
  kbdstroke_class = class_new(gensym("kbdstroke"),
    (t_newmethod)kbdstroke_new,
	0, sizeof(t_kbdstroke), 0, A_GIMME, 0);
  class_addmethod(kbdstroke_class, (t_method)kbdstroke_downkey, gensym("downkey"), A_GIMME, 0);
  class_addmethod(kbdstroke_class, (t_method)kbdstroke_upkey, gensym("upkey"), A_GIMME, 0);
  class_addmethod(kbdstroke_class, (t_method)kbdstroke_key, gensym("key"), A_GIMME, 0);
  class_addmethod(kbdstroke_class, (t_method)kbdstroke_mouse_move, gensym("mouse_move"), A_GIMME, 0);
  class_addmethod(kbdstroke_class, (t_method)kbdstroke_mouse_pos, gensym("mouse_pos"), A_GIMME, 0);
  class_addmethod(kbdstroke_class, (t_method)kbdstroke_mouse_wheel, gensym("mouse_wheel"), A_GIMME, 0);
class_addmethod(kbdstroke_class, (t_method) kbdstroke_get_screen_xsize, gensym("xsize"), A_GIMME, 0);

 class_addfloat(kbdstroke_class, (t_method)kbdstroke_float);
  
  post("==============================================");
  post("                Pure winAPI");
  post("Copyleft 2007 Patrice Colet");
  post("GNU Public License");
  post("[kbdstroke] simulates Windows keyboard events.");
  post("==============================================");
  
}
